import requests
import json
from bs4 import BeautifulSoup as bs
# import pandas as pd
import re
from time import sleep
from pytimeparse.timeparse import timeparse
from datetime import datetime, date, time, timedelta
from datasize import DataSize
import sqlite3
import os.path
from tqdm import tqdm

from .database import con
from .utils import filesize_string_to_bytes

# Store network urls, as well as common headers
networks = {
    'avistaz': {
        'sites': [
            {
                'name':     'avistaz',
                'url':      'https://avistaz.to',
                'cookie':    os.environ['AVISTAZ_COOKIE']
            },
            {
                'name':     'cinemaz',
                'url':      'https://cinemaz.to',
                'cookie':    os.environ['CINEMAZ_COOKIE']
            
            },
        ],
        'headers': {
            'authority': 'avistaz.to',
            'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9',
            'accept-encoding': 'gzip, deflate, br',
            'accept-language': 'en-AU,en;q=0.9,pl-PL;q=0.8,pl;q=0.7,en-US;q=0.6,en-GB;q=0.5',
            'sec-ch-ua': '" Not A;Brand";v="99", "Chromium";v="96", "Google Chrome";v="96"',
            'sec-ch-ua-mobile': '?0',
            'sec-ch-ua-platform': 'Linux',
            'sec-fetch-dest': 'document',
            'sec-fetch-mode': 'navigate',
            'sec-fetch-site': 'same-origin',
            'sec-fetch-user': '?1',
            'upgrade-insecure-requests': '1'
        }

    }
}


cur = con.cursor()

def generate_headers(network_headers, cookie, url):
    site_specific_headers = {
        'cookie': cookie,
        'referer': url
    }
    return {**network_headers,**site_specific_headers}



# History table functions
def download_history_page(site_url, network_headers, cookie):
    # Pull the torrent history, and insert cookies/url
    payload={}
    history_url = f"{site_url}/profile/superraiden/history" 
    headers = generate_headers(network_headers, cookie, history_url)

    res = requests.request("GET", history_url, headers=headers, data=payload)
    return res.text

def parse_history_table(history_page_text):
    # Parse the text we downloaded
    soup = bs(history_page_text, 'html.parser')

    # Create headers to be able to convert a row into a dictionary
    try:
        history_table = soup.find('table').find('tbody')
    except AttributeError as ex:
        print(history_page_text)
        raise ValueError("FAILURE ON history_table = soup.find('table').find('tbody')")
    header_names = [
        'Type', 'File', 'Download Link', 'Active', 'Completed', 'Up',
        'Down', 'Ratio', 'Added', 'Updated', 'Seed Duration', 'Hit and Run'
    ]

    # Create history dict
    torrent_history = []
    for row in soup.find('table').find('tbody').find_all('tr'):
        cells = row.find_all('td')
        torrent_row_info = {header_name: cells[i] for i, header_name in enumerate(header_names)}
        # torrent_history.append( torrent_row_info )
        torrent_url = torrent_row_info['File'].find('a')['href']

        result = {
            'torrent_id':           re.search(r"(\d+).+", torrent_url).groups()[0],
            'torrent_url':          torrent_url,
            'title':                torrent_row_info['File'].find('a')['title'],
            'file_size':            filesize_string_to_bytes(torrent_row_info['File'].find('span', {'title': 'File Size'}).text),

            'uploaded':             filesize_soup_to_bytes(torrent_row_info['Up'].find('span', {'class': 'text-green'})),
            'uploaded_credited':    filesize_soup_to_bytes(torrent_row_info['Up'].find('span', {'class': 'text-blue'})),

            'downloaded':           filesize_soup_to_bytes(torrent_row_info['Down'].find('span', {'class': 'text-red'})),
            'downloaded_credited':  filesize_soup_to_bytes(torrent_row_info['Down'].find('span', {'class': 'text-orange'})),

            'seeders':              int(torrent_row_info['File'].find('span', {'title': 'Seeders'}).text.strip()),
            'leechers':             int(torrent_row_info['File'].find('span', {'title': 'Leechers'}).text.strip()),
            'completed':            int(torrent_row_info['File'].find('span', {'title': 'Completed'}).text.strip()),
        }
        torrent_history.append(result)

    return torrent_history


# Torrent page stuff
def download_torrent_page(torrent_url, network_headers, cookie):
    # Pull the torrent history, and insert cookies/url
    payload={}
    headers = generate_headers(network_headers, cookie, torrent_url)

    res = requests.request("GET", torrent_url, headers=headers, data=payload)
    return res.text

def parse_torrent_page(torrent_page_text):
    soup = bs(torrent_page_text, 'html.parser')
    torrent_info_table = soup.find('table')

    torrent_info = {}
    rowz = torrent_info_table.find('tbody').find_all('tr', recursive=False)
    for i, rowz_raw in enumerate(rowz):
        cells = rowz_raw.find_all('td', recursive=False)
        torrent_info[cells[0].text] = cells[1]

    # Discounts
    discounts = []
    dying_torrent_seconds_left = 0
    dying_torrent_expiry = ''
    # Is row 1 showing any discounts
    if 'Discount' in torrent_info.keys():
        # These are our discounts
        discounts = [discount['title'] for discount in torrent_info['Discount'].find_all('i')]
        # Is it a dying torrent with an expiry?
        if len(torrent_info['Discount'].findAll('div', {'class': 'badge-extra'})) > 1:
            dying_torrent_seconds_left = timeparse(torrent_info['Discount'].find_all('td')[1].find('strong').text)
            dying_torrent_expiry = (datetime.now() + timedelta(seconds=468000)).strftime("%Y-%m-%d %H:%M:%S")


    web_result = {
        'info_hash':                    torrent_info['Info Hash'].text,
        'languages':                    [lang.text.strip() for lang in torrent_info['Audio'].findAll('span')],
        'discounts':                    discounts,
        'dying_torrent_seconds_left':   dying_torrent_seconds_left,
        'dying_torrent_expiry':         dying_torrent_expiry,
        'type':                         torrent_info['Type'].text.strip(),
        'uploaded_at':                  datetime.strptime(torrent_info['Uploaded at'].text.strip().split('\n')[0], '%d %b %Y %H:%M').strftime("%Y-%m-%d %H:%M:%S")


    }
    test = 1
    return web_result


# Utils
def filesize_soup_to_bytes(soup):
    if soup is None:
        return 0
    return filesize_string_to_bytes(soup.text)


# Main function

def pull_website_history():
    # 'Now' variable for load times
    today = datetime.now()
    today_string = today.strftime("%Y-%m-%d_%H:%M:%S")

    # Pull current data for each network/site
    for network_name, network in networks.items():
        for site in network['sites']:
            print(f"Downloading site {site['name']}")
            
            # Download the site's history page
            torrent_history = parse_history_table(
                download_history_page(
                    site['url'],
                    network['headers'],
                    site['cookie']
                )
            )

            # Download info for each torrent
            for torrent_history_row in tqdm(torrent_history):
                #Rate limit
                sleep(1)

                torrent_page_data = parse_torrent_page(
                    download_torrent_page(
                        torrent_history_row['torrent_url'],
                        network['headers'],
                        site['cookie']
                    )
                )

                # Merge data between the history table and the torrent's page
                # Add site and network data
                torrent_data = { **torrent_history_row, **torrent_page_data }
                # torrent_bk = f"{site['name']}/{torrent_data['torrent_id']}/{today_string}"
                
                # torrent_data['bk'] = f"{site['name']}/{torrent_data['torrent_id']}/{today_string}"
                # torrent_data['site_name'] = site['name']
                torrent_data['network_name'] = network_name

                data = [
                    (
                        site['name'],
                        torrent_data['torrent_id'],
                        network_name,
                        torrent_data['torrent_url'],
                        torrent_data['title'],
                        torrent_data['file_size'],
                        torrent_data['uploaded'],
                        torrent_data['uploaded_credited'],
                        torrent_data['downloaded'],
                        torrent_data['downloaded_credited'],
                        torrent_data['seeders'],
                        torrent_data['leechers'],
                        torrent_data['completed']
                    ),
                ]
                cur.executemany("""
                    INSERT INTO stg_torrent_website(
                        site,
                        torrent_id,
                        network_name,
                        torrent_url,
                        title,
                        file_size,
                        uploaded,
                        uploaded_credited,
                        downloaded,
                        downloaded_credited,
                        seeders,
                        leechers,
                        completed
                    ) VALUES(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)""", data)
                con.commit()  # Remember to commit the transaction after executing INSERT.

# pull_torrent_soups()

if __name__ == "__main__":
    pull_live_history()





#https://avistaz.to/rss/feed?fid=29186&pid=82292d24010149ab8d7d6fe373fdee09


import os
import sqlite3
import __main__
import sys

# db_filename = 'torrent_history.sqlite3.db'
# BASE_DIR = os.path.dirname(os.path.abspath(str(sys.modules['__main__'].__file__)))
# db_path = os.path.join(BASE_DIR, db_filename)
db_path = os.environ['SQLITE_DB']
# print(f"Using db file: {db_path}")

con = sqlite3.connect(db_path)



#!/bin/python
import sys

from pull_torrent_info.download_from_website import pull_website_history
from pull_torrent_info.download_from_transmission import pull_transmission_data
from pull_torrent_info.download_rss_feeds import pull_rss_feeds


if __name__ == "__main__":
    for function in sys.argv[1:]:
        if function in ('pull_live_history', 'pull_transmission_data', 'pull_rss_feeds'):
            print(globals())
            globals()[function]()


    print(sys.argv[1:])
    pull_website_history()
    # pull_transmission_data()
    # pull_rss_feeds()


import feedparser
import re
from datetime import datetime
from .utils import filesize_string_to_bytes, datestring_to_int, datestring_standardise
from .database import con
from tqdm import tqdm
import os

feed_urls = [
    {
        'name': 'avistaz',
        'filters': ['Free Download', 'Double Upload'],
        'url':  os.environ['AVISTAZ_RSS'],
    },
    {
        'name': 'cinemaz',
        'filters': ['Free Download', 'Double Upload'],
        'url':  os.environ['CINEMAZ_RSS'],
    },
]
cur = con.cursor()

def find_regex_entry(regex_code, entry, data_type):
    try:
        match = re.findall(regex_code, entry)[0]
        if data_type == 'filesize_int':
            return filesize_string_to_bytes(match)
        elif  data_type == 'date':
            return datestring_standardise(match, '%b %d, %Y')
        else:
            return match
    except IndexError:
        if data_type in ('filesize_int',  'date_int'):
            return 0
        else:
            return data_type

def pull_rss_feeds():
    today = datetime.now()
    today_string = today.strftime("%Y-%m-%d_%H:%M:%S")

    for feed_url in feed_urls:
        url_to_download = feed_url['url']
        site_name = feed_url['name']

        feed = feedparser.parse(url_to_download)

        regex = [
            ('type', '<strong>Type<\/strong>: (.+?)<br \/>', ''),
            ('size', '<strong>Size<\/strong>: (.+?)<br \/>', 'filesize_int'),
            ('uploaded', '<strong>Uploaded<\/strong>: (.+?)<br \/>', 'date'),
            ('seed', 'Seed<\/strong>: (\d+) \| ', 0),
            ('leech', 'Leech<\/strong>: (\d+) \| ', 0),
            ('completed', 'Completed<\/strong>: (\d+)', 0.0),
            ('site_url', 'href="(.+?)" title=', ''),
        ]

        # re.findall('<strong>File Name<\/strong>: (.+?)<br \/>', feed.entries[1].summary)[0]

        nice_results = []
        db_results = []


        for entry in tqdm(feed.entries):
            result = { regex_name: find_regex_entry(regex_code, entry.summary, data_type) for (regex_name, regex_code, data_type) in regex }

            result['title'] = entry.title
            result['torrent_url'] = entry.link
            result['author'] = entry.author
            result['published'] = datestring_standardise(entry.published, '%a, %d %b %Y %H:%M:%S %z')
            result['infohash'] = entry.infohash

            rss_bk = f"{result['infohash']}/{today_string}"
            result['bk'] = rss_bk      

            data = [
                (
                    result['torrent_url'],                
                    result['infohash'],
                    site_name,
                    '/'.join(feed_url['filters']),
                    result['type'],
                    result['size'],
                    result['uploaded'],
                    result['seed'],
                    result['leech'],
                    result['completed'],
                    result['site_url'],
                    result['title'],
                    result['author'],
                    result['published'],
                ),
            ]
            
            db_results.append(data)
            nice_results.append(result)


            cur.executemany("""
                INSERT INTO stg_torrent_rss(
                    torrent_url,
                    infohash,
                    site,
                    rss_filter,
                    media_type,
                    torrent_size,
                    date_uploaded,
                    seed_users,
                    leech_users,
                    completed,
                    site_url,
                    title,
                    author,
                    published
                ) VALUES(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)""", data)
            con.commit()  # Remember to commit the transaction after executing INSERT.



from datetime import datetime
from datasize import DataSize

def filesize_string_to_bytes(filesize_string):
    return int(DataSize(filesize_string.strip().replace(' ', '')))

def datestring_standardise(date_string, format):
    return datetime.strptime(date_string, format).strftime("%Y-%m-%d %H:%M:%S")

def datestring_to_int(date_string, format):
    # '%d %b %Y %H:%M'
    # '%b %d, %Y'
    return int(datetime.strptime(date_string, format).timestamp())
    pass



from clutch import Client
from pathlib import Path
from .database import con
from datetime import datetime
import json
import os

def safe_serialize(obj):
  default = lambda o: f"<<non-serializable: {type(o).__qualname__}>>"
  return json.dumps(obj, default=default, indent = 4)

def pull_transmission_data():
    print('Pulling data from transmission')

    today = datetime.now()
    today_string = today.strftime("%Y-%m-%d_%H:%M:%S")

    client = Client(
        address=os.environ['TRANSMISSION_URL'],
        username=os.environ['TRANSMISSION_USERNAME'],
        password=os.environ['TRANSMISSION_PASSWORD']
    )
    
    cur = con.cursor()

    response = client.torrent.accessor(all_fields=True)
    torrents = response.arguments.torrents
    
    for torrent in torrents:
        torrent_bk = f"{torrent.hash_string}/{today_string}"

        raw_data =  torrent.__dict__
        extra_data = {
                'torrent_bk' : torrent_bk,
                'snapshot_datetime': today,
        }
        data = raw_data | extra_data
        
        insert_row = [
            (
                data['hash_string'],
                data['activity_date'],
                data['added_date'],
                data['comment'],
                data['date_created'],
                data['downloaded_ever'],
                data['hash_string'],
                data['magnet_link'],
                data['name'],
                data['seconds_downloading'],
                data['seconds_seeding'],
                data['total_size'],
                data['upload_ratio']
            ),
        ]
        cur.executemany(
            """INSERT INTO stg_torrent_transmission(
                    filehash,
                    activity_date,
                    added_date,
                    comment,
                    date_created,
                    downloaded_ever,
                    hash_string,
                    magnet_link,
                    name,
                    seconds_downloading,
                    seconds_seeding,
                    total_size,
                    upload_ratio
            ) VALUES(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)""", insert_row)
        con.commit()  # Remember to commit the transaction after executing INSERT.